# -*- coding: utf-8 -*-
"""
:Author: Jaekyoung Kim
:Date: 2018. 9. 6.
"""
PORTFOLIO_RETURN = 'portfolio_return'
BENCHMARK_RETURN = 'benchmark_return'
ACTIVE_RETURN = 'active_return'
ACTIVE_RISK = 'active_risk'
SR = 'sharpe_ratio'
IR = 'information_ratio'
CAGR = 'compound_annual_growth_rate'
MDD = 'maximum_drawdown'
FAMA_FRENCH_ALPHA = 'Fama_French_alpha'
FAMA_FRENCH_ALPHA_P_VALUE = 'Fama_French_alpha_p_value'
FAMA_FRENCH_BETA = 'Fama_French_beta'
TURNOVER = 'turnover'
